{ Programm: winmenu.pas
 Funktion: Routinen fr die Entwicklung von Fenstern & Dialogboxen
           (Buttons, Rahmen, Eingabezeilen, Checkboxen und Radiobuttons)
           im VGA-Grafikmodus
 Sprache : Turbo Pascal ab Version 6.0
 Autor   : Alexander Grau }

unit WinMenu;

interface

const
  { Message box classes }

  mfWarning      = $0000;
  mfError        = $0001;
  mfInformation  = $0002;
  mfConfirmation = $0003;

  { Message box button flags }

  mfYesButton    = $0100;      
  mfNoButton     = $0200;      
  mfOKButton     = $0400;      
  mfCancelButton = $0800;      

  mfYesNoCancel  = mfYesButton + mfNoButton + mfCancelButton;
  mfOKCancel     = mfOKButton + mfCancelButton;

var
  ListBoxData: array[1..255] of string[20];

procedure OutGChar(XP, YP: integer; ch: Char);
procedure OutGTextXY(X, Y:Integer; Textstring: string);
procedure OutGText(Textstring: string);
procedure OutTextFmt(x1, y1, LineDist, width: integer; s: string);
function InRange(x, y, x1, y1, x2, y2: integer): boolean;
procedure CreateButton(x1,y1,x2,y2: integer; title: string;
                       high: boolean; pressed: boolean);
function ButtonDown(x1, y1, x2, y2: integer;
                    high: boolean; title: string;
                    PressButton: boolean): boolean;
procedure CreateWindow(x1, y1, x2, y2: integer; color, border: byte;
                       var p: pointer; var size: word);
procedure RestoreBackground(x, y: integer; var p: pointer; var size: word);
procedure CreateDialog(x1, y1, x2, y2: integer; color, border: byte;
                       title: string; var p: pointer; var size: word);
procedure CreateInputLine(x1, y1, x2, y2: integer; DlgData: string);
procedure wait(time: word);
function InputLineActive(no: byte; x1, y1, x2, y2: integer;
                         var DlgData: string): boolean;
procedure CreateGroup(x1, y1, x2, y2: integer; title: string);
procedure CreateCheckBox(x1,y1,x2,y2: integer;
                         title: string; active: boolean);
procedure CheckBoxActive(x1,y1,x2,y2: integer; var active: boolean);
procedure CreateRadioButton(x1,y1,x2,y2: integer;
                            title: string; number, active: byte);
procedure RadioButtonActive(x1,y1,x2,y2: integer;
                            number: byte; var value, oldvalue: byte);
procedure CreateVertScrollBar(x1, y1, x2, y2: integer);
procedure CreateListBox(x1, y1, x2, y2: integer; count: byte);
function ListBoxActive(x1, y1, x2, y2: integer;
                       count: byte; var filestart: byte): byte;
function MessageBox(Msg: string; params: pointer; AOptions: word): byte;
function MessageBoxRect(x1,y1,x2,y2: integer; msg: string; params: pointer;
  AOptions: word): byte;
function InputBox(title: string; ALabel: string; var s: string;
  limit: byte): boolean;
function InputBoxRect(x1,y1,x2,y2: integer; title: string; ALabel: string;
  var s: string; limit: byte): boolean;

implementation

uses crt, graph, dos, mouse, drivers;
const
  VSeg         =  $A000;
  IndexReg     =  $03ce;
  DatenReg     =  $03cf;
var
  i,x,y            : integer;
  buttons          : byte;
  WinBackground    : byte;
  ch               : char;
  cpu              : registers;
  FontBuf  : array[0..4095] of byte;
  SHLength : byte;          { Wert fr bitweises Rechts-/Linksschieben }
  filestart,count: byte;
  TimerTicks     : word;
  IBarActive     : array[1..5] of boolean;

procedure InitGraphFont;
var
  cpu: registers;
  Row: integer;
begin
  cpu.ax:=$1130;
  cpu.bh:=$6;
  intr($10,cpu);
  for Row:=0 to 4095 do
    FontBuf[Row]:=Mem[CPU.ES:CPU.BP+Row];
end;

procedure OutGChar(XP, YP: integer; ch: Char);
var
  cpu     : registers;
  x, y    : integer;
  Bitmap  : byte;
  CharBits: byte;
begin
  for y:=0 to 15 do
  begin
    Bitmap:=Mem[VSeg:YP*80+y*80+XP div 8];
    CharBits:=FontBuf[16*Ord(ch)+y];
    port[DatenReg]:=CharBits SHR SHLength;      { Bitmaske setzen }
    Mem[VSeg:YP*80+y*80+XP div 8]:=GetColor;
    if SHLength>0 then
    begin
      Bitmap:=Mem[VSeg:YP*80+y*80+XP div 8+1];
      port[DatenReg]:=CharBits SHL (8-SHLength);  { Bitmaske setzen }
      Mem[VSeg:YP*80+y*80+XP div 8+1]:=GetColor;
    end;
  end;
end;

procedure OutGTextXY(X, Y:Integer; Textstring: string);
var
  I: integer;
begin
  port[IndexReg]:=5;    { Modusregister }
  port[DatenReg]:=2;    { Schreibzugriff Modus 2 }
  SHLength:=X SHL 13 SHR 13;
  port[IndexReg]:=3;    { Manipulationsregister }
  port[DatenReg]:=0;
  port[IndexReg]:=8;    { Bitmasken-Register }
  for I:=1 to length(Textstring) do
    if X+I*8-8<633 then OutGChar(X+I*8-8, Y, Textstring[i]);
  MoveTo(X+length(Textstring)*8,Y);

  { Index- und Datenregister wieder auf Standardwerte setzen }
  port[IndexReg]:=8;     { Bitmasken-Register }
  port[DatenReg]:=$FF;   { Standardbitmaske (FFh) }
  port[IndexReg]:=5;     { Modusregister }
  port[DatenReg]:=0;
  port[IndexReg]:=3;     { Manipulationsregister }
  port[DatenReg]:=0;
end;

procedure OutGText(Textstring: string);
begin
  OutGTextXY(GetX, GetY, Textstring);
end;

procedure OutTextFmt(x1, y1, LineDist, width: integer; s: string);
var
  CharPos,devide: byte;
  center: boolean;
  x,y: integer;
  c: char;
  temp: string;
begin
  CharPos:=1; 
  center:=false;
  x:=x1; y:=y1;
  temp:='';
  while CharPos<=length(s) do
  begin
    c:=s[CharPos];
    case c of
      #13: begin
             if NOT center then OutTextXY(x,y,temp)
             else begin
               OutTextXY(x+(width-length(temp)*8) SHR 1,y,temp);
               center:=false;
             end;
             inc(y,LineDist);
             x:=x1;
             temp:='';
           end;
      #10: begin
             x:=x1;
           end;
      #3: begin
            if temp='' then center:=true
              else temp:=temp+c;
          end;
      else temp:=temp+c;
    end;
    inc(CharPos);

    if length(temp)*8+8>width then
    begin
      devide:=0;
      for i:=1 to length(temp) do
        if (temp[i]=#32) OR (temp[i]='-') then devide:=i;
      if devide=0 then devide:=length(temp);
      dec(CharPos,length(temp)-devide);
      delete(temp,devide+1,length(temp)-devide);
      if NOT center then OutTextXY(x,y,temp)
      else begin
        OutTextXY(x+(width-length(temp)*8) SHR 1,y,temp);
        center:=false;
      end;
      inc(y,LineDist);
      x:=x1;
      temp:='';
    end;

  end;
  if NOT center then OutTextXY(x,y,temp)
    else OutTextXY(x+(width-length(temp)*8) SHR 1,y,temp);
end;

Function InRange(x, y, x1, y1, x2, y2: integer): boolean;
begin
  if (x>=x1) and (x<=x2) and (y>=y1) and (y<=y2) then
    InRange:=true
  else
    InRange:=false
end;

procedure CreateButton(x1,y1,x2,y2: integer; title: string;
                       high: boolean; pressed: boolean);
var
  OldStyle: TextSettingsType;
  flag    : boolean;
  position: byte;
  ShortKey: byte;
begin
  GetTextSettings(OldStyle);
  SetLineStyle(SolidLn,0,NormWidth);
  SetTextStyle(DefaultFont,HorizDir,1);
  SetTextJustify(1, CenterText);
  DisableMouse;
  if NOT pressed then
  begin
    SetColor(black);
    line(x1+1,y1,x2-1,y1); line(x2,y1+1,x2,y2-1);
    line(x1+1,y2,x2-1,y2); line(x1,y2-1,x1,y1+1);
    SetColor(white);
    line(x1+1,y1+1,x2-1,y1+1); line(x1+1,y1+1,x1+1,y2-1);
    if high then
    begin
      line(x1+2,y1+2,x2-1,y1+2); line(x1+2,y1+2,x1+2,y2-1);
    end;
    SetColor(darkgray);
    line(x1+1,y2-1,x2-1,y2-2); line(x1+2,y2-2,x2-1,y2-1);
    line(x2-1,y1+1,x2-1,y2-1); line(x2-2,y1+2,x2-2,y2-1);
    SetFillStyle(SolidFill,lightgray);
    if high then bar(x1+3,y1+3,x2-3,y2-3)
      else bar(x1+2,y1+2,x2-3,y2-3);
    SetColor(black);
    flag:=false;
    position:=0;
    if pos('~',title)>0 then ShortKey:=16 else ShortKey:=8;
    for i:=1 to length(title) do
    begin
      if title[i]='~' then
      begin
        if flag then SetColor(black) else SetColor(lightred);
        flag:=true;
      end
      else begin
        OutTextXY(x1+((x2-x1)-(TextWidth(title)-ShortKey)) div 2
                  +position*8,y1+(y2-y1) div 2,title[i]);
        inc(position);
      end;
    end;
  end
  else begin
    SetColor(lightgray);
    line(x1+1,y2-1,x2-1,y2-2); line(x1+2,y2-2,x2-1,y2-1);
    line(x2-1,y1+1,x2-1,y2-1); line(x2-2,y1+2,x2-2,y2-1);
    SetColor(darkgray);
    line(x1+1,y1+1,x2-1,y1+1); line(x1+1,y1+1,x1+1,y2-1);
    line(x1+2,y1+2,x2-1,y1+2); line(x1+2,y1+2,x1+2,y2-1);
    SetFillStyle(SolidFill,lightgray);
    bar(x1+3,y1+3,x2-3,y2-3);
    SetColor(black);
    flag:=false;
    position:=0;
    if pos('~',title)>0 then ShortKey:=16 else ShortKey:=8;
    for i:=1 to length(title) do
    begin
      if title[i]='~' then
      begin
        if flag then SetColor(black) else SetColor(lightred);
        flag:=true;
      end
      else begin
        OutTextXY(x1+((x2-x1)-(TextWidth(title)-ShortKey)) div 2
                  +position*8+2,y1+(y2-y1) div 2+2,title[i]);
        inc(position);
      end;
    end;
  end;
  EnableMouse;
  with OldStyle do
  begin
    SetTextJustify(Horiz, Vert);
    SetTextStyle(Font, Direction, CharSize);
  end;
end;

function ButtonDown(x1, y1, x2, y2: integer;
                    high: boolean; title: string;
                    PressButton: boolean): boolean;
begin
  ButtonDown:=false;
  ReadMouse(x,y,buttons);
  if (InRange(x, y, x1, y1, x2, y2) AND (buttons<>0)) then
  begin
    CreateButton(x1,y1,x2,y2,title,high,true);
    if PressButton then
      while (buttons<>0) AND (InRange(x, y, x1, y1, x2, y2)) do
        ReadMouse(x,y,buttons);
    CreateButton(x1,y1,x2,y2,title,high,false);
    if InRange(x, y, x1, y1, x2, y2) then ButtonDown:=true;
  end;
end;

procedure CreateWindow(x1, y1, x2, y2: integer; color, border: byte;
                       var p: pointer; var size: word);
begin
  Size := ImageSize(x1, y1, x2, y2);
  DisableMouse;
  if MaxAvail >= Size then
  begin
    GetMem(P, Size);
    GetImage(x1, y1, x2, y2, P^);
  end else P:=nil;
  WinBackground:=color;
  SetFillStyle(SolidFill, WinBackground);
  Bar(x1+1, y1+1, x2-1, y2-1);
  SetColor(black);
  SetLineStyle(SolidLn,0,NormWidth);
  Rectangle(x1, y1, x2, y2);
  SetColor(border);
  for i:=1 to 3 do Rectangle(x1+i, y1+i, x2-i, y2-i);
  EnableMouse;
end;

procedure RestoreBackground(x, y: integer; var p: pointer; var size: word);
begin
   if P<>nil then
   begin
     DisableMouse;
     PutImage(x, y, P^, NormalPut);
     EnableMouse;
     FreeMem(p, Size);
   end;
end;

procedure CreateDialog(x1, y1, x2, y2: integer; color, border: byte;
                       title: string; var p: pointer; var size: word);
var
  OldStyle: TextSettingsType;
begin
  WinBackground:=color;
  GetTextSettings(OldStyle);
  CreateWindow(x1, y1, x2, y2, WinBackground, border, p, size);
  DisableMouse;
  SetLineStyle(SolidLn,0,NormWidth);
  SetFillStyle(SolidFill, border);
  bar(x1+5,y1+5,x2-5,y1+19);
  SetColor(border);
  line(x1+5,y1+20,x2-5,y1+20);
  SetColor(white);
  SetTextJustify(CenterText, CenterText);
  OutTextXY(x1+(x2-x1) div 2,y1+13,title);
  EnableMouse;
  with OldStyle do
  begin
    SetTextJustify(Horiz, Vert);
    SetTextStyle(Font, Direction, CharSize);
  end;
end;

procedure CreateInputLine(x1, y1, x2, y2: integer; DlgData: string);
var
  OldStyle: TextSettingsType;
begin
  GetTextSettings(OldStyle);
  DisableMouse;
  SetColor(black);
  SetTextJustify(0, CenterText);
  SetLineStyle(SolidLn,0,NormWidth);
  Rectangle(x1,y1,x2,y2);
  OutTextXY(x1+3,y1+(y2-y1+1) div 2,DlgData);
  EnableMouse;
  with OldStyle do
  begin
    SetTextJustify(Horiz, Vert);
    SetTextStyle(Font, Direction, CharSize);
  end;
end;

{$F+}
procedure TimerInt; interrupt;
begin
  if TimerTicks>0 then dec(TimerTicks);
end;
{$F-}

procedure Wait(time: word);
var
  Int1CSave: pointer;
begin
  TimerTicks:=time;
  GetIntVec($1c,Int1CSave);
  SetIntVec($1c,@TimerInt);
  while TimerTicks<>0 do;
  SetIntVec($1c,Int1CSave);
end;

function InputLineActive(no: byte; x1, y1, x2, y2: integer;
                         var DlgData: string): boolean;
var
  OldStyle: TextSettingsType;
  NewData : string;
  visible : boolean;
  Int1CSave: pointer;

procedure ChangeMarker;
begin
  if visible then
  begin
    visible:=false;
    SetColor(WinBackground)
  end else
  begin
    visible:=true;
    SetColor(black);
  end;
  Line(x1+x+2, y1+1, x1+x+2, y2-1);
end;

begin
  InputLineActive:=false;
  ReadMouse(x,y,buttons);
  if InRange(x, y, x1, y1, x2, y2) then
  begin
    if buttons=0 then begin
      DefineGraphCursor(IBar);
      IBarActive[no]:=true;
      exit;
    end;
  end else begin
    if IBarActive[no] then begin
      DefineGraphCursor(StandardShape);
      IBarActive[no]:=false;
    end;
    exit;
  end;
  NewData:=DlgData;
  TimerTicks:=0; x:=0; ch:=#0;
  GetTextSettings(OldStyle);
  SetTextJustify(0, CenterText);
  DisableMouse;
  GetIntVec($1c,Int1CSave);
  SetIntVec($1c,@TimerInt);
  repeat
    if KeyPressed then
    begin
      ch:=ReadKey;
      case ch of
        #0: begin
              ch:=ReadKey;
              if visible then ChangeMarker;
              case ch of
                #75: if x>=8 then dec(x,8);
                #77: if x<length(NewData)*8 then inc(x,8);
                #71: if x>0 then x:=0;
                #79: if x<length(NewData)*8 then x:=length(NewData)*8;
              end;
            end;
        #8: begin
              if x>=8 then
              begin
                if visible then ChangeMarker;
                SetColor(WinBackground);
                OutTextXY(x1+x-5, y1+(y2-y1+1) div 2,NewData[x div 8]);
                Delete(NewData,length(NewData),1);
                dec(x,8);
              end;
            end;
        #32..#255: begin
                     if x1+x+8<x2 then
                     begin
                       if visible then ChangeMarker;
                       if NewData[x div 8+1]<>'' then
                       begin
                         SetColor(WinBackground);
                         OutTextXY(x1+x+3, y1+(y2-y1+1) div 2,
                         NewData[x div 8+1]);
                       end;
                       if length(NewData)<x div 8+1 then
                         NewData:=NewData+#32;
                       NewData[x div 8+1]:=ch;
                       SetColor(black);
                       OutTextXY(x1+x+3, y1+(y2-y1+1) div 2,ch);
                       inc(x,8);
                     end;
                   end;
      end;
    end else
    if TimerTicks=0 then
    begin
      ChangeMarker;
      TimerTicks:=10;
    end;
  until (ch=#27) OR (ch=#13);
  SetIntVec($1c,Int1CSave);
  if visible then ChangeMarker;
  if ch<>#27 then
  begin
    InputLineActive:=true;
    DlgData:=NewData;
  end
  else begin
    SetColor(WinBackground);
    OutTextXY(x1+3,y1+(y2-y1+1) div 2,NewData);
    SetColor(black);
    OutTextXY(x1+3,y1+(y2-y1+1) div 2,DlgData);
  end;
  EnableMouse;
  with OldStyle do
  begin
    SetTextJustify(Horiz, Vert);
    SetTextStyle(Font, Direction, CharSize);
  end;
end;

procedure CreateGroup(x1, y1, x2, y2: integer; title: string);
begin
  SetColor(black);
  SetLineStyle(SolidLn,0,NormWidth);
  DisableMouse;
  Rectangle(x1,y1,x2,y2);
  SetColor(WinBackground);
  line(x1+4,y1,x1+TextWidth(title)+10,y1);
  SetColor(black);
  OutTextXY(x1+8,y1,title);
  EnableMouse;
end;

procedure CreateCheckBox(x1,y1,x2,y2: integer;
                         title: string; active: boolean);
var
  OldStyle: TextSettingsType;
begin
  GetTextSettings(OldStyle);
  SetColor(black);
  SetLineStyle(SolidLn,0,NormWidth);
  DisableMouse;
  rectangle(x1,y1,x1+y2-y1,y2);
  if active then
  begin
    line(x1+1, y1+1, x1+y2-y1-1, y2-1);
    line(x1+y2-y1-1, y1+1, x1+1, y2-1);
  end;
  SetTextJustify(0, CenterText);
  OutTextXY(x2-TextWidth(title),y1+(y2-y1) div 2,title);
  EnableMouse;
  with OldStyle do
  begin
    SetTextJustify(Horiz, Vert);
    SetTextStyle(Font, Direction, CharSize);
  end;
end;

procedure CheckBoxActive(x1,y1,x2,y2: integer; var active: boolean);
begin
  ReadMouse(x,y,buttons);
  if (InRange(x, y, x1, y1, x2, y2) AND (buttons<>0)) then
  begin
    if active then SetColor(WinBackground) else SetColor(black);
    DisableMouse;
    line(x1+1, y1+1, x1+y2-y1-1, y2-1);
    line(x1+y2-y1-1, y1+1, x1+1, y2-1);
    EnableMouse;
    active:=NOT active;
    while (buttons<>0) do
    begin
      ReadMouse(x,y,buttons);
      while KeyPressed do ReadKey;
    end;
  end;
end;

procedure CreateRadioButton(x1,y1,x2,y2: integer;
                            title: string; number, active: byte);
var
  OldStyle: TextSettingsType;
begin
  GetTextSettings(OldStyle);
  SetColor(black);
  DisableMouse;
  circle(x1+(y2-y1) div 2, y1+(y2-y1) div 2, (y2-y1) div 2);
  SetFillStyle(SolidFill, black);
  if number=active then
    PieSlice(x1+(y2-y1) div 2, y1+(y2-y1) div 2, 0, 360, (y2-y1) div 2-2);
  SetTextJustify(0, CenterText);
  OutTextXY(x2-TextWidth(title),y1+(y2-y1) div 2,title);
  EnableMouse;
  with OldStyle do
  begin
    SetTextJustify(Horiz, Vert);
    SetTextStyle(Font, Direction, CharSize);
  end;
end;

procedure RadioButtonActive(x1,y1,x2,y2: integer;
                            number: byte; var value, oldvalue: byte);
begin
  if number=oldvalue then
  begin
    SetColor(WinBackground);
    SetFillStyle(SolidFill, WinBackground);
    oldvalue:=0;
    DisableMouse;
    PieSlice(x1+(y2-y1) div 2, y1+(y2-y1) div 2, 0, 360, (y2-y1) div 2-2);
    EnableMouse;
    while (buttons<>0) do
    begin
      ReadMouse(x,y,buttons);
      while KeyPressed do ReadKey;
    end;
  end;
  ReadMouse(x,y,buttons);
  if (InRange(x, y, x1, y1, x2, y2) AND (buttons<>0)) then
  begin
    if number<>value then
    begin
      SetColor(black);
      SetFillStyle(SolidFill, black);
      oldvalue:=value;
      value:=number;
      DisableMouse;
      PieSlice(x1+(y2-y1) div 2, y1+(y2-y1) div 2, 0, 360, (y2-y1) div 2-2);
      EnableMouse;
    end;
  end;
end;

procedure CreateVertScrollBar(x1, y1, x2, y2: integer);
var
  OldStyle: TextSettingsType;
begin
  GetTextSettings(OldStyle);
  SetColor(black);
  DisableMouse;
  Rectangle(x1, y1, x2, y2);
  CreateButton(x1, y1, x2, y1+16,#24,false,false);
  CreateButton(x1, y2-16, x2, y2,#25,false,false);
  SetFillStyle(SolidFill, LightGray);
  Bar(x1+1, y1+17, x2-1, y2-17);
  EnableMouse;
  with OldStyle do
  begin
    SetTextJustify(Horiz, Vert);
    SetTextStyle(Font, Direction, CharSize);
  end;
end;

procedure CreateListBox(x1, y1, x2, y2: integer; count: byte);
var
  OldStyle: TextSettingsType;
  row: byte;
begin
  GetTextSettings(OldStyle);
  SetColor(black);
  DisableMouse;
  Rectangle(x1, y1, x2, y2);
  CreateButton(x2-15, y1, x2, y1+16,#24,false,false);
  CreateButton(x2-15, y2-16, x2, y2,#25,false,false);
  line(x2-15,y2-16,x2-15,y1+16);
  SetFillStyle(SolidFill, LightGray);
  Bar(x2-14, y1+17, x2-1, y2-17);
  SetFillStyle(SolidFill, white);
  bar(x1+1,y1+1,x2-16,y2-1);
  if count>(y2-y1+1) div 11 then count:=(y2-y1+1) div 11;
  for row:=1 to count do
    OutTextXY(x1+8,y1+row*11-8,ListBoxData[row]);
  EnableMouse;
  with OldStyle do
  begin
    SetTextJustify(Horiz, Vert);
    SetTextStyle(Font, Direction, CharSize);
  end;
end;

function ListBoxActive(x1, y1, x2, y2: integer;
                       count: byte; var filestart: byte): byte;
var
  OldStyle: TextSettingsType;
  row: byte;
  active: byte;
  x_old, y_old: integer;
begin
  active:=0;
  if ((ButtonDown(x2-15, y1, x2, y1+16,false,#24,false) AND (filestart>0))
    OR (ButtonDown(x2-15, y2-16, x2, y2,false,#25,false) AND
      (filestart<count-(y2-y1+1) div 11)))
        AND (count>(y2-y1+1) div 11) then
  begin
    GetTextSettings(OldStyle);
    SetFillStyle(SolidFill, white);
    SetColor(black);
    repeat
      if InRange(x, y, x2-15, y1, x2, y1+16) AND (filestart>0) then
      dec(filestart)
        else if InRange(x, y, x2-15, y2-16, x2, y2)
          AND (filestart<count-(y2-y1+1) div 11) then inc(filestart);
      DisableMouse;
      for row:=1 to (y2-y1+1) div 11 do
      begin
        bar(x1+1, y1+row*11-10, x2-16, y1+row*11);
        OutTextXY(x1+8,y1+row*11-8,ListBoxData[row+filestart]);
      end;
      EnableMouse;
      ReadMouse(x,y,buttons);
      if buttons<>0 then Wait(1);
    until buttons=0;
    with OldStyle do
    begin
      SetTextJustify(Horiz, Vert);
      SetTextStyle(Font, Direction, CharSize);
    end;
  end

  else
  if InRange(x, y, x1, y1+1, x2-16, y2-1) AND (buttons<>0) then
  begin
    repeat
      DisableMouse;
      for row:=1 to (y2-y1+1) div 11 do
        if InRange(x, y, x1, y1+row*11-10, x2-16, y1+row*11) then
        begin
          active:=row+filestart;
          setcolor(white);
          SetFillStyle(SolidFill,blue);
          bar(x1+1, y1+row*11-10, x2-16, y1+row*11);
          OutTextXY(x1+8,y1+row*11-8,ListBoxData[row+filestart]);
        end else
        begin
          SetFillStyle(SolidFill,white);
          setcolor(black);
          bar(x1+1, y1+row*11-10, x2-16, y1+row*11);
          OutTextXY(x1+8,y1+row*11-8,ListBoxData[row+filestart]);
        end;
      EnableMouse;
      x_old:=x; y_old:=y;
      repeat
        ReadMouse(x,y,buttons);
      until (x<>x_old) OR (y<>y_old) OR (buttons=0);
    until NOT (InRange(x, y, x1+1, y1+1, x2-16, y2-1) AND (buttons<>0));
  end;
  ListBoxActive:=active;
end;

function MessageBox(Msg: string; params: pointer; AOptions: word): byte;
begin
  MessageBox:=MessageBoxRect(130,150,509,329, msg, params, AOptions);
end;

function MessageBoxRect(x1,y1,x2,y2: integer; msg: string; params: pointer;
  AOptions: word): byte;
const
  ButtonName: array[0..3] of string[7] =
    ('~J~a', '~N~ein', 'O~K~', 'Abbruch');
  titles: array[0..3] of string[11] =
    ('Warnung','Fehler','Information','Besttigen');
var
  x,i: integer;
  s,temp: string;
  p: pointer;
  size: word;
  ButtonCount: byte;
  ButtonList: array[0..3] of byte;
  result: byte;
  ext: boolean;
  OldStyle: TextSettingsType;
begin
  GetTextSettings(OldStyle);
  SetLineStyle(SolidLn,0,NormWidth);
  SetTextStyle(DefaultFont,HorizDir,1);
  SetTextJustify(LeftText, BottomText);
  CreateDialog(x1,y1,x2,y2,white,blue,titles[AOptions AND $3],p,size);
  FormatStr(s, msg, params^);
  SetColor(black);
  DisableMouse;
  OutTextFmt(x1+30, y1+50, 16, x2-x1-59, msg);
  EnableMouse;
  ButtonCount:=0;
  for i:=0 to 3 do
    if AOptions AND ($0100 SHL i) <> 0 then
    begin
      ButtonList[ButtonCount]:=i;
      inc(ButtonCount);
    end;
  x:=x1+((x2-x1+1)-(ButtonCount*80+(ButtonCount-1)*20)) SHR 1;
  for i:=0 to ButtonCount-1 do
  begin
    CreateButton(x, y2-44, x+79, y2-24,
    ButtonName[ButtonList[i]], true, false);
    inc(x,100);
  end;

  repeat
    result:=0;
    x:=x1+((x2-x1+1)-(ButtonCount*80+(ButtonCount-1)*20)) SHR 1;
    for i:=0 to ButtonCount-1 do
    begin
      if ButtonDown(x, y2-44, x+79, y2-24, true,
      ButtonName[ButtonList[i]], true) then result:=i+1;
      inc(x,100);
    end;
    if KeyPressed then
    begin
      ch:=ReadKey;
      if ch=#0 then
      begin
        ch:=ReadKey;
        ext:=true;
      end else ext:=false;
      for i:=0 to ButtonCount-1 do
      case ext of
        true : case ch of
                 #36: if ButtonList[i]=0 then result:=i+1;
                 #49: if ButtonList[i]=1 then result:=i+1;
                 #37: if ButtonList[i]=2 then result:=i+1;
               end;
        false: case ch of
                 #13: if ButtonList[i]=2 then result:=i+1;
                 #27: if ButtonList[i]=3 then result:=i+1;
               end;
      end;
    end;
  until result<>0;

  RestoreBackground(x1,y1,p,size);
  MessageBoxRect:=result;
  with OldStyle do
  begin
    SetTextJustify(Horiz, Vert);
    SetTextStyle(Font, Direction, CharSize);
  end;
end;

function InputBox(title: string; ALabel: string; var s: string;
  limit: byte): boolean;
begin
  InputBox:=InputBoxRect(110,170,529,309, title, ALabel, s, limit);
end;

function InputBoxRect(x1,y1,x2,y2: integer; title: string; ALabel: string;
  var s: string; limit: byte): boolean;
var
  p: pointer;
  size: word;
  result: byte;
  temp: string;
  OldStyle: TextSettingsType;
begin
  GetTextSettings(OldStyle);
  SetLineStyle(SolidLn,0,NormWidth);
  SetTextStyle(DefaultFont,HorizDir,1);
  SetTextJustify(LeftText, BottomText);
  CreateDialog(x1,y1,x2,y2,white,blue,title,p,size);
  SetColor(black);
  DisableMouse;
  OutTextXY(x1+20,y1+48,ALabel);
  EnableMouse;
  CreateInputLine(x1+30+length(ALabel)*8, y1+35,
    x1+30+length(ALabel)*8+limit*8, y1+55, s);
  CreateButton(x2-180, y2-50, x2-110, y2-30, 'OK', true, false);
  CreateButton(x2-90, y2-50, x2-20, y2-30, 'Abbruch', true, false);
  temp:=s; result:=0;
  repeat
    InputLineActive(1,x1+30+length(ALabel)*8, y1+35,
      x1+30+length(ALabel)*8+limit*8, y1+55,temp);
    if ButtonDown(x2-180, y2-50, x2-110, y2-30, true,'OK', true) then
      result:=1;
    if ButtonDown(x2-90, y2-50, x2-20, y2-30, true,'Abbruch', true) then
      result:=2;
    if KeyPressed then
    case ReadKey of
      #13: result:=1;
      #27: result:=2;
    end;
  until result<>0;
  if result=1 then
  begin
    InputBoxRect:=true;
    s:=temp;
  end else InputBoxRect:=false;
  RestoreBackground(x1,y1,p,size);
  with OldStyle do
  begin
    SetTextJustify(Horiz, Vert);
    SetTextStyle(Font, Direction, CharSize);
  end;
end;


begin
  InitGraphFont;
end.
